@@ -70,14 +70,22 @@ bool TitlebarWindowManagerPolicy::handle_pointer_event(MirPointerEvent const* ev
         {
             if (auto const target = tools.window_at(old_cursor))
             {
-                tools.select_active_window(target);
-                tools.drag_active_window(cursor - old_cursor);
+                if (tools.select_active_window(target) == target)
+                    tools.drag_active_window(cursor - old_cursor);
             }
             consumes_event = true;
         }
 
         if (mir_pointer_event_button_state(event, mir_pointer_button_tertiary))
         {
+            {   // Workaround for lp:1627697
+                auto now = std::chrono::steady_clock::now();
+                if (resizing && now < last_resize+std::chrono::milliseconds(20))
+                    return true;
+
+                last_resize = now;
+            }
+
             if (!resizing)
                 tools.select_active_window(tools.window_at(old_cursor));
             is_resize_event = resize(tools.active_window(), cursor, old_cursor);
@@ -94,19 +102,45 @@ bool TitlebarWindowManagerPolicy::handle_pointer_event(MirPointerEvent const* ev
                 if (possible_titlebar.application() == titlebar_provider->session())
                 {
                     auto const& info = tools.info_for(possible_titlebar);
-                    tools.select_active_window(info.parent());
-                    tools.drag_active_window(cursor - old_cursor);
+                    if (tools.select_active_window(info.parent()) == info.parent())
+                        tools.drag_active_window(cursor - old_cursor);
                     consumes_event = true;
                 }
             }
         }
     }
+    
+    if (resizing && !is_resize_event)
+        end_resize();
 
     resizing = is_resize_event;
     old_cursor = cursor;
     return consumes_event;
 }
 
+void TitlebarWindowManagerPolicy::end_resize()
+{
+    if (!resizing  && !pinching)
+        return;
+
+    if (auto window = tools.active_window())
+    {
+        auto& window_info = tools.info_for(window);
+
+        auto new_size = window.size();
+        auto new_pos  = window.top_left();
+        window_info.constrain_resize(new_pos, new_size);
+
+        WindowSpecification modifications;
+        modifications.top_left() = new_pos;
+        modifications.size() = new_size;
+        tools.modify_window(window_info, modifications);
+    }
+
+    resizing = false;
+    pinching = false;
+}
+
 bool TitlebarWindowManagerPolicy::handle_touch_event(MirTouchEvent const* event)
 {
     auto const count = mir_touch_event_point_count(event);
@@ -182,23 +216,31 @@ bool TitlebarWindowManagerPolicy::handle_touch_event(MirTouchEvent const* event)
                 auto const delta_width = DeltaX{touch_pinch_width - old_touch_pinch_width};
                 auto const delta_height = DeltaY{touch_pinch_height - old_touch_pinch_height};
 
-                auto const delta_x = DeltaX{touch_pinch_left - old_touch_pinch_left};
-                auto const delta_y = DeltaY{touch_pinch_top - old_touch_pinch_top};
-
-                auto const new_width = std::max(old_size.width + delta_width, Width{5});
-                auto const new_height = std::max(old_size.height + delta_height, Height{5});
-
-                auto new_pos = window.top_left() + delta_x + delta_y;
-                Size new_size{new_width, new_height};
+                auto new_width = std::max(old_size.width + delta_width, Width{5});
+                auto new_height = std::max(old_size.height + delta_height, Height{5});
+                Displacement delta{
+                    DeltaX{touch_pinch_left - old_touch_pinch_left},
+                    DeltaY{touch_pinch_top  - old_touch_pinch_top}};
 
                 auto& window_info = tools.info_for(window);
+                keep_size_within_limits(window_info, delta, new_width, new_height);
 
-                window_info.constrain_resize(new_pos, new_size);
+                auto new_pos = window.top_left() + delta;
+                Size new_size{new_width, new_height};
+
+                {   // Workaround for lp:1627697
+                    auto now = std::chrono::steady_clock::now();
+                    if (pinching && now < last_resize+std::chrono::milliseconds(20))
+                        return true;
+
+                    last_resize = now;
+                }
 
                 WindowSpecification modifications;
                 modifications.top_left() = new_pos;
                 modifications.size() = new_size;
                 tools.modify_window(window_info, modifications);
+                pinching = true;
             }
             consumes_event = true;
         }
@@ -209,6 +251,9 @@ bool TitlebarWindowManagerPolicy::handle_touch_event(MirTouchEvent const* event)
             tools.select_active_window(window);
     }
 
+    if (!consumes_event && pinching)
+        end_resize();
+
     old_cursor = cursor;
     old_touch_pinch_top = touch_pinch_top;
     old_touch_pinch_left = touch_pinch_left;
@@ -226,13 +271,22 @@ void TitlebarWindowManagerPolicy::advise_new_window(WindowInfo const& window_inf
     if (application == titlebar_provider->session())
     {
         titlebar_provider->advise_new_titlebar(window_info);
-        return;
+
+        auto const parent = window_info.parent();
+
+        if (tools.active_window() == parent)
+            titlebar_provider->paint_titlebar_for(parent, 0xFF);
+        else
+            titlebar_provider->paint_titlebar_for(parent, 0x3F);
     }
+}
 
-    if (application == spinner.session() || !window_info.needs_titlebar(window_info.type()))
-        return;
+void TitlebarWindowManagerPolicy::handle_window_ready(WindowInfo& window_info)
+{
+    if (window_info.window().application() != spinner.session() && window_info.needs_titlebar(window_info.type()))
+        titlebar_provider->create_titlebar_for(window_info.window());
 
-    titlebar_provider->create_titlebar_for(window_info.window());
+    CanonicalWindowManagerPolicy::handle_window_ready(window_info);
 }
 
 void TitlebarWindowManagerPolicy::advise_focus_lost(WindowInfo const& info)
@@ -285,6 +339,9 @@ bool TitlebarWindowManagerPolicy::handle_keyboard_event(MirKeyboardEvent const*
     auto const scan_code = mir_keyboard_event_scan_code(event);
     auto const modifiers = mir_keyboard_event_modifiers(event) & modifier_mask;
 
+    if (action != mir_keyboard_action_repeat)
+        end_resize();
+
     if (action == mir_keyboard_action_down && scan_code == KEY_F11)
     {
         switch (modifiers)
@@ -407,7 +464,7 @@ void TitlebarWindowManagerPolicy::toggle(MirSurfaceState state)
         WindowSpecification modifications;
 
         modifications.state() = (info.state() == state) ? mir_surface_state_restored : state;
-
+        tools.place_and_size_for_state(modifications, info);
         tools.modify_window(info, modifications);
     }
 }
@@ -450,6 +507,22 @@ bool TitlebarWindowManagerPolicy::resize(Window const& window, Point cursor, Poi
     auto new_width = old_pos.size.width + x_sign * delta.dx;
     auto new_height = old_pos.size.height + y_sign * delta.dy;
 
+    keep_size_within_limits(window_info, delta, new_width, new_height);
+
+    Size new_size{new_width, new_height};
+    Point new_pos = top_left + left_resize*delta.dx + top_resize*delta.dy;
+
+    WindowSpecification modifications;
+    modifications.top_left() = new_pos;
+    modifications.size() = new_size;
+    tools.modify_window(window, modifications);
+
+    return true;
+}
+
+void TitlebarWindowManagerPolicy::keep_size_within_limits(
+    WindowInfo const& window_info, Displacement& delta, Width& new_width, Height& new_height) const
+{
     auto const min_width  = std::max(window_info.min_width(), Width{5});
     auto const min_height = std::max(window_info.min_height(), Height{5});
 
@@ -467,16 +540,22 @@ bool TitlebarWindowManagerPolicy::resize(Window const& window, Point cursor, Poi
             delta.dy = DeltaY{0};
     }
 
-    Size new_size{new_width, new_height};
-    Point new_pos = top_left + left_resize*delta.dx + top_resize*delta.dy;
-    window_info.constrain_resize(new_pos, new_size);
+    auto const max_width  = window_info.max_width();
+    auto const max_height = window_info.max_height();
 
-    WindowSpecification modifications;
-    modifications.top_left() = new_pos;
-    modifications.size() = new_size;
-    tools.modify_window(tools.info_for(window), modifications);
+    if (new_width > max_width)
+    {
+        new_width = max_width;
+        if (delta.dx < DeltaX{0})
+            delta.dx = DeltaX{0};
+    }
 
-    return true;
+    if (new_height > max_height)
+    {
+        new_height = max_height;
+        if (delta.dy < DeltaY{0})
+            delta.dy = DeltaY{0};
+    }
 }
 
 WindowSpecification TitlebarWindowManagerPolicy::place_new_surface(
